module.exports = {
  "name": "pl_PL",
  "day": {
    "abbrev": [
      "nie",
      "pon",
      "wto",
      "śro",
      "czw",
      "pią",
      "sob"
    ],
    "full": [
      "niedziela",
      "poniedziałek",
      "wtorek",
      "środa",
      "czwartek",
      "piątek",
      "sobota"
    ]
  },
  "month": {
    "abbrev": [
      "sty",
      "lut",
      "mar",
      "kwi",
      "maj",
      "cze",
      "lip",
      "sie",
      "wrz",
      "paź",
      "lis",
      "gru"
    ],
    "full": [
      "styczeń",
      "luty",
      "marzec",
      "kwiecień",
      "maj",
      "czerwiec",
      "lipiec",
      "sierpień",
      "wrzesień",
      "październik",
      "listopad",
      "grudzień"
    ]
  },
  "meridiem": [
    "",
    ""
  ],
  "date": "%d.%m.%Y",
  "time24": "%T",
  "dateTime": "%a, %-d %b %Y, %T",
  "time12": "",
  "full": "%a, %-d %b %Y, %T %Z"
}
