module.exports = [require("./Aden.js"),require("./Almaty.js"),require("./Amman.js"),require("./Anadyr.js"),require("./Aqtau.js"),require("./Aqtobe.js"),require("./Ashgabat.js"),require("./Ashkhabad.js"),require("./Atyrau.js"),require("./Baghdad.js"),require("./Bahrain.js"),require("./Baku.js"),require("./Bangkok.js"),require("./Barnaul.js"),require("./Beirut.js"),require("./Bishkek.js"),require("./Brunei.js"),require("./Calcutta.js"),require("./Chita.js"),require("./Choibalsan.js"),require("./Chongqing.js"),require("./Chungking.js"),require("./Colombo.js"),require("./Dacca.js"),require("./Damascus.js"),require("./Dhaka.js"),require("./Dili.js"),require("./Dubai.js"),require("./Dushanbe.js"),require("./Famagusta.js"),require("./Gaza.js"),require("./Harbin.js"),require("./Hebron.js"),require("./Ho_Chi_Minh.js"),require("./Hong_Kong.js"),require("./Hovd.js"),require("./Irkutsk.js"),require("./Istanbul.js"),require("./Jakarta.js"),require("./Jayapura.js"),require("./Jerusalem.js"),require("./Kabul.js"),require("./Kamchatka.js"),require("./Karachi.js"),require("./Kashgar.js"),require("./Kathmandu.js"),require("./Katmandu.js"),require("./Khandyga.js"),require("./Kolkata.js"),require("./Krasnoyarsk.js"),require("./Kuala_Lumpur.js"),require("./Kuching.js"),require("./Kuwait.js"),require("./Macao.js"),require("./Macau.js"),require("./Magadan.js"),require("./Makassar.js"),require("./Manila.js"),require("./Muscat.js"),require("./Nicosia.js"),require("./Novokuznetsk.js"),require("./Novosibirsk.js"),require("./Omsk.js"),require("./Oral.js"),require("./Phnom_Penh.js"),require("./Pontianak.js"),require("./Pyongyang.js"),require("./Qatar.js"),require("./Qostanay.js"),require("./Qyzylorda.js"),require("./Rangoon.js"),require("./Riyadh.js"),require("./Saigon.js"),require("./Sakhalin.js"),require("./Samarkand.js"),require("./Seoul.js"),require("./Shanghai.js"),require("./Singapore.js"),require("./Srednekolymsk.js"),require("./Taipei.js"),require("./Tashkent.js"),require("./Tbilisi.js"),require("./Tehran.js"),require("./Tel_Aviv.js"),require("./Thimbu.js"),require("./Thimphu.js"),require("./Tokyo.js"),require("./Tomsk.js"),require("./Ujung_Pandang.js"),require("./Ulaanbaatar.js"),require("./Ulan_Bator.js"),require("./Urumqi.js"),require("./Ust-Nera.js"),require("./Vientiane.js"),require("./Vladivostok.js"),require("./Yakutsk.js"),require("./Yangon.js"),require("./Yekaterinburg.js"),require("./Yerevan.js")]
