module.exports = [require("./Antananarivo.js"),require("./Chagos.js"),require("./Christmas.js"),require("./Cocos.js"),require("./Comoro.js"),require("./Kerguelen.js"),require("./Mahe.js"),require("./Maldives.js"),require("./Mauritius.js"),require("./Mayotte.js"),require("./Reunion.js")]
